gjChipLocator <- read.table("gjChipLocator.txt", header=T)

head(gjChipLocator)

## gjChipLocator is a data frame that holds information about the 330 Munsell chips:

## 1. their number (the chips were presented to the test persons in this order)
## 2. their Munsell code (letters for rows and numbers for columns)
## 3./4. their numerical coordinates on the Munsell chart
## 5.-7. their RGB color values

munsell.colors <- rgb(gjChipLocator[,5:7],maxColorValue=255)
# rgb transforms rgb color code into the R-internal color representation
# munsell.colors is a vector of color codes of length 300
# it holds the color codes of the Munsell chips, ordered according to the chip numbers

plot.new() # opens a new canvas

rect(0,0,1,1,col="red")

rect(0,0,1,1,col=munsell.colors[1])

rect(0,0,1,1,col=munsell.colors[2])

rect(0,0,1,1,col=munsell.colors[3])

rect(0,0,1,1,col=munsell.colors[4])


chips <- data.frame(gjChipLocator,row.names=1)
munsellChart <- function(c) {
  plot.new()
  rect(chips[,2]/42,chips[,3]/10,(chips[,2]+1)/42,
       (chips[,3]+1)/10,col=c)
}
# this function takes a vector c of 330 colors as argument
# and draws a Munsell chart with the chips colored according
# to c

munsellChart(munsell.colors)

############
# 3d plot
############

library(rgl)

cie <- read.table("cnum-vhcm-lab-new.txt",header=T)
cie <- cie[order(cie$cnum),c(8,9,7)]
rownames(cie) <- 1:330

# in theory the following should work, but somehow it doesn't with the
# version that is currently installed on my computer:
plot3d(cie[,c(2,3,1)],col=munsell.colors,type="s",
       size=2,box=F,axes=F,xlab="",ylab="",zlab="")


# here is a workaround that does the job, if slowly
open3d()
plot3d(cie,type="n",box=F,axes=F,xlab="",ylab="",zlab="")
for (i in 1:330) {
  spheres3d(cie[i,],col=munsell.colors[i],type="s",radius=5)  
}


#####################
# visualizing the Berlin & Kay data
#####################

bk.foci <- read.table("gjBK-foci.txt",header=T)

bk.foci <- merge(bk.foci,gjChipLocator[,1:2],by.x="foci",by.y="mName")
# adding chip numbers

bk.fDensity <- table(factor(bk.foci$cnum,1:330))
# creates a table which counts how often each chip has been named as
# the focus of some color in some language
# First converting foci$cnum from a numerical vector into a factor
# is necessary to ensure that 0-counts are included in the table

plot(sort(bk.fDensity,decreasing=T),log="xy")
# frequency distribution: it roughly follows a power law

bk.fDensity.grey <- grey(1 - bk.fDensity/max(bk.fDensity))
# converts numbers into shades of grey. High counts are converted
# to black and low counts to white

munsellChart(bk.fDensity.grey) # visualizes fDensity

bkTerm <- na.exclude(read.table("BK-term.txt"))

colnames(bkTerm) <- c("lnum","snum","cnum","term")

languageNames <- c("Arabic","Bahasa Indonesia","Bulgarian",
                   "Cantonese", "Catalan", "English",
                   "Hebrew", "Hungarian", "Ibibo", "Japanese",
                   "Korean", "Mandarin", "Mexican Spanish",
                   "Pomo", "Swahili", "Tagalog", "Thai", "Tzeltal",
                   "Urdu", "Vietnamese")

mycolors = c('white','black','red','green','blue', 'yellow',
  'brown', 'RosyBrown', 'cyan','DarkBlue','DarkOliveGreen3',
  'magenta', 'MediumVioletRed','MistyRose2','orange4','RoyalBlue',
  'salmon3', 'SkyBlue','violet',
  'SlateBlue', 'PeachPuff')


# visualizing the extension of Arabic color terms

bkTerm.Arabic <- bkTerm[bkTerm$lnum==1,]

terms.Arabic <- unique(bkTerm.Arabic[,4])

bkTerm.Arabic[,4] <- factor(bkTerm.Arabic[,4],terms.Arabic)

partition.Arabic <- rep(0,330)

partition.Arabic[bkTerm.Arabic[,3]] <- as.numeric(bkTerm.Arabic[,4])+1

munsellChart(mycolors[partition.Arabic+1])


# doing the same for all 20 languages
# and storing the result in a pdf file

for (l in 1:20) {
  lname <- as.character(l)
  if (l<10) {
    lname <- paste("0",lname,sep="") # make sure that lname is a two-digit character string
  }
  bkTermL <- bkTerm[bkTerm[,1]==l,]
  terms <- unique(bkTermL[,4])
  bkTermL[,4] <- factor(bkTermL[,4],terms)
  partition <- rep(0,330)
  partition[bkTermL[,3]] <- as.numeric(bkTermL[,4])+1
  pdf(paste("bk_l",lname,".pdf",sep=""),height=4,width=10)
  munsellChart(mycolors[partition+1])
  title(languageNames[l])
  dev.off()
}

###################
#
#  World Color Survey
#
###################

foci <- read.table("foci-gj.txt",header=T)

head(foci)

foci <- merge(foci,gjChipLocator[,1:2],by.x="mName",by.y="mName")
# adding chip numbers

fDensity <- table(factor(foci$cnum,1:330))
# creates a table which counts how often each chip has been named as
# the focus of some color in some language
# First converting foci$cnum from a numerical vector into a factor
# is necessary to ensure that 0-counts are included in the table

plot(sort(fDensity,decreasing=T),log="xy")
# frequency distribution: it roughly follows a power law


par(bg="grey") # set the background color to grey
barplot(sort(fDensity,decreasing=T),log="y",
        col=munsell.colors[order(fDensity,decreasing=T)],
        xlab="Munsell chips", ylab="# named as focal color",
        main="Distribution of focal colors",border="NA",names.arg="")
# plot the frequency distribution of focal colors across all participants
# y-axis is logarithmically scaled

fDensity.log <- log(fDensity) # logarithm makes the distribution flatter
fDensity.log <- fDensity.log-min(fDensity.log) # normalize distribution (minimum is set to 0)
fDensity.log <- fDensity.log/max(fDensity.log) # normalize distribution (maximum set to 1)

par(bg="white")
munsellChart(grey(1-fDensity.log)) # visualizes fDensity.log

displayDensity <- function(v) {
  v <- v-min(v)
  v <- v/max(v)
  munsellChart(grey(1-v))
}

# here is a different visualization

# remove the values for grey
fDensity.ordered <- fDensity[order(gjChipLocator$x,gjChipLocator$y)]
#order the density values according to the position of the corresponding chips

fDensity.matrix <- t(matrix(fDensity.ordered[11:330], # remove the grey chips
                            ncol=40))
# coerce the values int a matrix

# contour map
image(log(fDensity.matrix)-mean(log(fDensity.matrix)),col=terrain.colors(100))
contour(log(fDensity.matrix),add=T)

par(mfrow=c(2,1))

image(log(fDensity.matrix)-mean(log(fDensity.matrix)),col=terrain.colors(100))
contour(log(fDensity.matrix),add=T)

displayDensity(log(log(fDensity)))

par(mfrow=c(1,1))

## the extension of color categories

term.table <- read.table("gjterm.txt", header=T)

head(term.table)

dim(term.table)

## This data base contains many missing values. They are indicated by a * or ?
## We have to remove them before we can work with it
## To be on the safe side, we remove all speakers that supplied incomplete data.


# extend the data frame with a column that uniquely identifies each speaker
term.table$ls <- paste(term.table$language,term.table$speaker,sep=".")

#identify the speaker that use "*" or "?" as terms ("speakers With Missing Data")
speakers.wmd <- unique(term.table[term.table$term %in% c("*","?"),]$ls)

#remove speakers with missing data
term.table <- term.table[!(term.table$ls %in% speakers.wmd),]

dim(term.table)
# note that we threw out more than 20% of the data. The data frame is still quite big though

#construct the vector of all remaining speakers
speakers <- unique(term.table$ls)

#pick a random speaker
random.speaker <- sample(speakers,1)

# extract the information about this speaker from term.table
extensions <- term.table[term.table$ls == random.speaker,]

# we are only interested in the terms that this particular speaker used
terms <- unique(extensions$term)
extensions$term <- factor(extensions$term,unique(terms))

# display the categorization data for this speaker
munsellChart(mycolors[as.numeric(extensions$term)+1])
# as mycolors[1] is "white", we add 1 to each color code


# to get an impression of the raw data, it is worthwhile to
# look at the categorizations of many speakers
while (T) {
  random.speaker <- sample(speakers,1)
  extensions <- term.table[term.table$ls == random.speaker,]
  terms <- unique(extensions$term)
  extensions$term <- factor(extensions$term,unique(terms))
  munsellChart(mycolors[as.numeric(extensions$term)+1])
  Sys.sleep(3)
}

# intra-linguistic variation

l <- sample(unique(term.table$language),1)
print(l)
l.speakers <- unique(term.table[term.table$language==l,]$ls)
terms.l <- unique(term.table[term.table$language==l,]$term)

for (s in l.speakers) {
  extensions <- term.table[term.table$ls == s,]
  extensions$term <- factor(extensions$term,unique(terms.l))
  munsellChart(mycolors[as.numeric(extensions$term)+1])
  Sys.sleep(3)
}

# create a contingency table terms/chips

term.table$tl <- paste(term.table$term,term.table$language,sep=".")

contingency.table <- table(term.table$tl,term.table$chip)
colnames(contingency.table) <- gjChipLocator$mName

# count the number of speakers that use each term

term.nspeaker <- table(unique(term.table[,c("ls","tl")])$tl)

# normalize contingeny.table by
# - dividing each row by the number of speaker that used the corresponding term
# - duplicate each row by this amount
# be warned: this may take up to 20 minutes!!!
# to save your cpu the effort, I stored the result in the file "normalizedCtable.txtx"


normalized.ctable <- NULL
for (tl in rownames(contingency.table)) {
  n <- term.nspeaker[tl]
  for (i in 1:n) {
    normalized.ctable <- rbind(normalized.ctable,contingency.table[tl,]/n)
  }
}

write.table(normalized.ctable,"normalizedCtable.txt")

normalized.ctable <- read.table("normalizedCtable.txt",header=T)

pca.terms <- prcomp(normalized.ctable,center=T,scale=F)

summary(pca.terms)

plot(pca.terms)

barplot(pca.terms$sdev[1:30]^2/sum(pca.terms$sdev^2),
        xlab="principal components",ylab="proportion of variance explained",
        col=c(rep("black",15),rep("white",15)))

# pca.terms$rotation holds the principal components (as columns)

displayDensity(pca.terms$rotation[,1])

displayDensity(pca.terms$rotation[,2])

displayDensity(pca.terms$rotation[,3])

displayDensity(pca.terms$rotation[,4])

displayDensity(pca.terms$rotation[,5])

displayDensity(pca.terms$rotation[,6])

displayDensity(pca.terms$rotation[,7])

displayDensity(pca.terms$rotation[,8])

displayDensity(pca.terms$rotation[,9])

displayDensity(pca.terms$rotation[,10])

displayDensity(pca.terms$rotation[,11])

displayDensity(pca.terms$rotation[,12])

displayDensity(pca.terms$rotation[,13])

displayDensity(pca.terms$rotation[,14])

displayDensity(pca.terms$rotation[,15])


# rotation the 15-dimensional subspace via the varimax algorithm
varimax.terms <- varimax(pca.terms$rotation[,1:15],normalize=F)

#varimax.terms$rotation holds the new coordinates (as columns)

displayDensity(varimax.terms$loadings[,1])

displayDensity(-varimax.terms$loadings[,2])

displayDensity(varimax.terms$loadings[,3])

displayDensity(varimax.terms$loadings[,4])

displayDensity(-varimax.terms$loadings[,5])

displayDensity(varimax.terms$loadings[,6])

displayDensity(varimax.terms$loadings[,7])

displayDensity(-varimax.terms$loadings[,8])

displayDensity(varimax.terms$loadings[,9])

displayDensity(varimax.terms$loadings[,10])

displayDensity(-varimax.terms$loadings[,11])

displayDensity(varimax.terms$loadings[,12])

displayDensity(-varimax.terms$loadings[,13])

displayDensity(-varimax.terms$loadings[,14])

displayDensity(varimax.terms$loadings[,15])

# projecting vectors to the lower-dimensional subspace

subspace <- pca.terms$rotation[,1:15]

remove.noise <- subspace %*% t(subspace)

ctable.cleaned <- as.matrix(normalized.ctable) %*% remove.noise

# cluster analysis
library(cluster)

clusters.2 <- clara(t(ctable.cleaned),2)
munsellChart(mycolors[clusters.2$cluster])

clusters.3 <- clara(t(ctable.cleaned),3)
munsellChart(mycolors[clusters.3$cluster])

clusters.4 <- clara(t(ctable.cleaned),4)
munsellChart(mycolors[clusters.4$cluster])

clusters.5 <- clara(t(ctable.cleaned),5)
munsellChart(mycolors[clusters.5$cluster])

clusters.6 <- clara(t(ctable.cleaned),6)
munsellChart(mycolors[clusters.6$cluster])

clusters.7 <- clara(t(ctable.cleaned),7)
munsellChart(mycolors[clusters.7$cluster])

clusters.8 <- clara(t(ctable.cleaned),8)
munsellChart(mycolors[clusters.8$cluster])

# hierarchical clustering
distance.matrix <- dist(t(ctable.cleaned))

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=2)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=3)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=4)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=5)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=6)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=7)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=8)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=9)])

munsellChart(mycolors[cutree(hclust(distance.matrix,method="ward"),k=10)])
